const data = {
  name: 'Energy Infrastructure',
  children: [
    {
      name: 'Power Plants',
      value: 2,
      type: 'blue',
      level: 'green',
      children: [
        {
          name: 'Coal Power Plant',
          value: 1,
          type: 'red',
          level: '#863a9f',
        },
        {
          name: 'Solar Power Plant',
          value: 1,
          type: 'red',
          level: '#863a9f',
        },
      ],
    },
    {
      name: 'Substations',
      value: 3,
      type: 'blue',
      level: 'green',
      children: [
        {
          name: 'Transmission Substation',
          value: 1,
          type: 'red',
          level: '#863a9f',
        },
        {
          name: 'Distribution Substation',
          value: 1,
          type: 'red',
          level: '#863a9f',
        },
        {
          name: 'Industrial Zone Substation',
          value: 1,
          type: 'red',
          level: '#863a9f',
        },
      ],
    },
    {
      name: 'Distribution Centers',
      value: 3,
      type: 'blue',
      level: 'green',
      children: [
        {
          name: 'Urban Area Distribution Center',
          value: 1,
          type: 'red',
          level: '#863a9f',
        },
        {
          name: 'Rural Area Distribution Center',
          value: 1,
          type: 'red',
          level: '#863a9f',
        },
        {
          name: 'Commercial District Distribution Center',
          value: 1,
          type: 'red',
          level: '#863a9f',
        },
      ],
    },
  ],
};
// Define width, height, and margin of the tree
const margin = {
  top: 20,
  right: 300,
  bottom: 200,
  left: 200,
};
const width = 850 - margin.left - margin.right;
const height = 850 - margin.top - margin.bottom;

const tree = d3.tree().size([height, width]);

// Assign hierarchical structure to classify the data
let nodes = d3.hierarchy(data, (node) => node.children);

// Provide data connection for nodes to their respective position in the tree
nodes = tree(nodes);

// Check if SVG already exists
const svgExists = d3.select('svg').empty();

// Add an svg element if it doesn't exist
if (svgExists) {
  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
    .style('background-color', 'lightblue'); // Set the background color here

  const g = svg
    .append('g') // appends group
    .attr(
      'transform',
      'translate(' + margin.left + ',' + margin.top + ')',
    );

  // Define links between nodes
  const link = g
    .selectAll('.link')
    .data(nodes.descendants().slice(1))
    .enter()
    .append('path')
    .attr('class', 'link')
    .style('stroke', (d) => d.data.level)
    .attr('d', (d) => {
      return (
        'M' +
        d.y +
        ',' +
        d.x +
        'C' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.x +
        ' ' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.parent.x +
        ' ' +
        d.parent.y +
        ',' +
        d.parent.x
      );
    });

  // Declare a group for each node present in the tree
  const node = g
    .selectAll('.node')
    .data(nodes.descendants())
    .enter()
    .append('g')
    .attr(
      'class',
      (d) =>
        'node' +
        (d.children ? ' node--internal' : ' node--leaf'),
    )
    .attr(
      'transform',
      (d) => 'translate(' + d.y + ',' + d.x + ')',
    );

  // Define shape of the node
  node
    .append('circle')
    .attr('r', (d) => d.data.value)
    .style('stroke', (d) => d.data.type)
    .style('fill', (d) => d.data.level);

  // Add a new node to the list
  node
    .append('text')
    .attr('dy', '.100em')
    .attr('x', (d) =>
      d.children
        ? (d.data.value + 5) * -1
        : d.data.value + 5,
    )
    .attr('y', (d) =>
      d.children && d.depth !== 0 ? -(d.data.value + 5) : d,
    )
    .style('text-anchor', (d) =>
      d.children ? 'end' : 'start',
    )
    .text((d) => d.data.name);
}
